@@ -1,6 +1,6 @@
 /*
  *  hand display - bridge hand widget for GTK+
- *  Copyright (C) 2007 Christoph Berg <cb@df7cb.de>
+ *  Copyright (C) 2007, 2008 Christoph Berg <cb@df7cb.de>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -17,15 +17,20 @@
 #include <glib/gtypes.h>
 #include <gtk/gtk.h>
 #include <math.h>
+#include <string.h>
 
 #include "hand_display.h"
 #include "int_int_marshal.h"
 
 /* static data */
 
-static int render_init = 0, card_width = 80, card_height = 0;
-static GdkPixbuf *card_pixbuf[53];
+static int render_init = 0, card_width = 100, card_height = 0;
+static gchar *svg_filename = NULL;
+static GdkPixbuf *card_pixbuf[52];
+static cairo_surface_t *grey_surface = NULL;
 static GtkWidget *drag_win = NULL; /* current drag icon */
+static GtkTargetEntry target_entry[1] = { { "card", 0, 0 } };
+static GtkTargetList *target_list = NULL;
 
 static char *suit_str[] = {"♣", "♦", "♥", "♠"};
 static char *rank_str[] = {"2", "3", "4", "5", "6", "7", "8", "9", "10", "J", "Q", "K", "A"};
@@ -74,13 +79,15 @@ render_card_init (char *card_fname)
 {
 	int i;
 	if (render_init) {
-		for (i = 0; i < 53; i++)
+		for (i = 0; i < 52; i++)
 			g_object_unref (card_pixbuf[i]);
+		cairo_surface_destroy (grey_surface);
 		render_init = 0;
 	}
 
 	GError *error = NULL;
-	GdkPixbuf *pb = gdk_pixbuf_new_from_file_at_size (card_fname, card_width * 13, 500, &error);
+	GdkPixbuf *pb = gdk_pixbuf_new_from_file_at_size (card_fname,
+			card_width * 13, card_width * 2 * 5, &error);
 	if (!pb) {
 		printf ("%s: %s.\n", card_fname, error->message);
 		return;
@@ -92,22 +99,23 @@ render_card_init (char *card_fname)
 	for (i = 0; i < 52; i++) {
 		card_pixbuf[i] = gdk_pixbuf_new (GDK_COLORSPACE_RGB, TRUE, 8, card_width, card_height);
 		if (!card_pixbuf[i]) {
-			printf ("moo: card_pixbuf[i]\n");
+			printf ("%s: rendering card_pixbuf failed\n", card_fname);
 			return;
 		}
 		int col = (i + 1) % 13;
 		int row = i / 13;
 		gdk_pixbuf_copy_area (pb, buf_width * col / 13.0, buf_height * row / 5.0,
+		//gdk_pixbuf_copy_area (pb, card_width * col, card_height * row,
 			card_width, card_height, card_pixbuf[i], 0, 0);
 	}
 	g_object_unref (pb);
 
-	card_pixbuf[52] =
-		gdk_pixbuf_new_from_file_at_size ("/usr/share/tenace/grey.svg", card_width, card_height, &error);
-	if (!card_pixbuf[52]) {
-		printf ("/usr/share/tenace/grey.svg: %s.\n", error->message);
-		return;
-	}
+	/* construct a alpha channel in card shape for greying out cards */
+	grey_surface = cairo_image_surface_create (CAIRO_FORMAT_A8, card_width, card_height);
+	cairo_t *ct = cairo_create (grey_surface);
+	gdk_cairo_set_source_pixbuf (ct, card_pixbuf[0], 0, 0);
+	cairo_paint_with_alpha (ct, 0.3);
+	cairo_destroy (ct);
 
 	render_init = 1;
 }
@@ -117,14 +125,14 @@ render_card (cairo_t *cr, int x, int y, int c, int color)
 {
 	if (!render_init)
 		return;
-
 	assert (0 <= c && c < 52);
-	
+
 	gdk_cairo_set_source_pixbuf (cr, card_pixbuf[c], x, y);
-	cairo_paint_with_alpha (cr, 1.0);
-	if (color == HAND_DISPLAY_OLD_CARD) {
-		gdk_cairo_set_source_pixbuf (cr, card_pixbuf[52], x, y);
-		cairo_paint_with_alpha (cr, 0.5);
+	cairo_paint (cr);
+
+	if (color == HAND_DISPLAY_OLD_CARD) { /* grey out old card */
+		cairo_set_source_rgb (cr, HAND_DISPLAY_GREY_FONT);
+		cairo_mask_surface (cr, grey_surface, x, y);
 	}
 	return;
 }
@@ -135,7 +143,7 @@ render_card (cairo_t *cr, int x, int y, int c, int color)
 static void
 draw (GtkWidget *hand, cairo_t *cr)
 {
-	int l, r, t, b, x, y;
+	int x, y;
 	HandDisplay *handdisp = HAND_DISPLAY(hand);
 	cairo_text_extents_t extents;
 	cairo_font_extents_t fextents;
@@ -250,39 +258,41 @@ draw (GtkWidget *hand, cairo_t *cr)
 			snprintf (cs, 6, "%s%s", suit_str[suit], rank_str[rank]);
 			cairo_text_extents (cr, cs, &extents);
 
-#define XOFF 18
-#define YOFF 20
-			switch (handdisp->table_seat[i]) { /* lower left point */
-				case 1: x = hand->allocation.width / 2 - extents.width - XOFF;
-				/*W*/	y = (hand->allocation.height + extents.height) / 2;
+#define XOFF 38
+#define YOFF 23
+#define WIDTH 52
+#define HEIGHT 28
+#define RAD 5
+			switch (handdisp->table_seat[i]) { /* middle point */
+				case 1: x = hand->allocation.width / 2 - XOFF;
+				/*W*/	y = hand->allocation.height / 2;
 					break;
-				case 2: x = (hand->allocation.width - extents.width) / 2;
+				case 2: x = hand->allocation.width / 2;
 				/*N*/	y = hand->allocation.height / 2 - YOFF;
 					break;
 				case 3: x = hand->allocation.width / 2 + XOFF;
-				/*E*/	y = (hand->allocation.height + extents.height) / 2;
+				/*E*/	y = hand->allocation.height / 2;
 					break;
-				case 4: x = (hand->allocation.width - extents.width) / 2;
-				/*S*/	y = hand->allocation.height / 2 + extents.height + YOFF;
+				case 4: x = hand->allocation.width / 2;
+				/*S*/	y = hand->allocation.height / 2 + YOFF;
 					break;
 				default:
 					return; /* stop here */
 			}
 
+			cairo_new_path (cr);
+			cairo_arc_negative (cr, x-WIDTH/2+RAD, y+HEIGHT/2 - RAD, RAD, M_PI, M_PI_2);
+			cairo_arc_negative (cr, x+WIDTH/2-RAD, y+HEIGHT/2 - RAD, RAD, M_PI_2, 0.0);
+			cairo_arc_negative (cr, x+WIDTH/2-RAD, y-HEIGHT/2 + RAD, RAD, 0.0, -M_PI_2);
+			cairo_arc_negative (cr, x-WIDTH/2+RAD, y-HEIGHT/2 + RAD, RAD, -M_PI_2, M_PI);
+			cairo_close_path (cr);
 			cairo_set_source_rgb (cr, HAND_DISPLAY_FOCUS_BG);
-			cairo_rectangle (cr, x + extents.x_bearing - XPAD,
-					y - YPAD + extents.y_bearing - 1,
-					extents.width + 2 * XPAD + 5,
-					extents.height + 2 * YPAD);
-			cairo_fill (cr);
+			cairo_fill_preserve (cr);
 			cairo_set_source_rgb (cr, HAND_DISPLAY_FONT);
-			cairo_rectangle (cr, x + extents.x_bearing - XPAD,
-					y - YPAD + extents.y_bearing - 1,
-					extents.width + 2 * XPAD + 5,
-					extents.height + 2 * YPAD);
 			cairo_stroke (cr);
 
-			cairo_move_to (cr, x, y);
+			cairo_move_to (cr, x - extents.width / 2 - 3,
+					y + extents.height / 2 - 3);
 			FONT_SYMBOL;
 			cairo_set_source_rgb (cr, suit_color[suit][0],
 				suit_color[suit][1], suit_color[suit][2]);
@@ -310,7 +320,9 @@ draw (GtkWidget *hand, cairo_t *cr)
 
 	/* "cards" style */
 
-	if (handdisp->style == HAND_DISPLAY_STYLE_CARDS) {
+	if (handdisp->style == HAND_DISPLAY_STYLE_CARDS && handdisp->mode != HAND_DISPLAY_MODE_HAND_X) {
+		/* we do not support MODE_X here, yet we still allow setting
+		 * STYLE_CARDS there to have the right drag icon */
 		y = MAX (hand->allocation.height - card_height - 5, 15);
 		int n = 0;
 		int suit;
@@ -334,7 +346,8 @@ draw (GtkWidget *hand, cairo_t *cr)
 						continue;
 
 					render_card (cr, x, yy, c, handdisp->cards[c]);
-					if (handdisp->cards[c] == HAND_DISPLAY_HILIGHT_CARD) {
+
+					if (handdisp->cards[c] == HAND_DISPLAY_HILIGHT_CARD) { /* draw triangular arrow */
 						cairo_set_source_rgb (cr, HAND_DISPLAY_HILIGHT_FONT);
 						cairo_move_to (cr, x + 3 + (hand->allocation.width - card_width - 10) / 24.0, yy);
 						cairo_rel_line_to (cr, -5, -5);
@@ -361,7 +374,6 @@ draw (GtkWidget *hand, cairo_t *cr)
 					cairo_restore (cr);
 				}
 			}
-			/* we do not support MODE_X here */
 		}
 
 		return;
@@ -499,6 +511,14 @@ hand_display_motion (GtkWidget *hand, GdkEventMotion *event)
 {
 	HandDisplay *handdisp = HAND_DISPLAY(hand);
 	int card = which_card(handdisp, event->x, event->y);
+	if (handdisp->drag && handdisp->cur_click >= 0 && handdisp->cur_click < 52 &&
+		gtk_drag_check_threshold (hand, handdisp->drag_x, handdisp->drag_y, event->x, event->y))
+	{
+		if (! target_list)
+			target_list = gtk_target_list_new (target_entry, 1);
+		handdisp->cur_drag = handdisp->cur_click;
+		gtk_drag_begin (hand, target_list, GDK_ACTION_COPY, 1, (GdkEvent *) event);
+	}
 	if (handdisp->cur_focus != card) {
 		if (handdisp->cur_focus != -1) {
 			redraw_card (hand, handdisp->cur_focus);
@@ -534,14 +554,21 @@ hand_display_button_press (GtkWidget *hand, GdkEventButton *event)
 
 	if (event->type != GDK_BUTTON_RELEASE) /* don't trigger after dnd */
 		handdisp->cur_focus = card;
-	if (event->type == GDK_BUTTON_PRESS)
+	if (event->type == GDK_BUTTON_PRESS) {
 		handdisp->cur_click = card;
+		if (handdisp->drag) {
+			handdisp->drag_x = event->x;
+			handdisp->drag_y = event->y;
+		}
+	}
 	if (handdisp->cur_focus == -1)
 		return FALSE;
 	redraw_card (hand, card);
 
-	if (event->type == GDK_BUTTON_RELEASE && handdisp->cur_click == card) {
-		g_signal_emit_by_name (handdisp, "card-clicked", card);
+	if (event->type == GDK_BUTTON_RELEASE) {
+		if (handdisp->cur_click == card)
+			g_signal_emit_by_name (handdisp, "card-clicked", card);
+		handdisp->cur_click = -1;
 	}
 	return FALSE;
 }
@@ -625,12 +652,10 @@ hand_display_size_allocate (GtkWidget *hand, GtkAllocation *allocation)
 /* http://live.gnome.org/GnomeLove/DragNDropTutorial */
 
 static void
-hand_display_drag_begin (GtkWidget *hand, GdkDragContext *dc, gpointer data)
+hand_display_drag_begin (GtkWidget *hand, GdkDragContext *dc, gpointer data /* unused */)
 {
 	HandDisplay *handdisp = HAND_DISPLAY (hand);
-	handdisp->cur_drag = handdisp->cur_click;
-	if (handdisp->cur_drag < 0)
-		return;
+	assert (handdisp->cur_drag >= 0 && handdisp->cur_drag < 52);
 	handdisp->cards[handdisp->cur_drag] |= HAND_DISPLAY_INVISIBLE_CARD;
 	handdisp->cur_click = -1;
 	redraw_card (hand, handdisp->cur_drag);
@@ -642,7 +667,7 @@ hand_display_drag_begin (GtkWidget *hand, GdkDragContext *dc, gpointer data)
 	}
 	drag_win = gtk_window_new (GTK_WINDOW_POPUP);
 	GtkWidget *card = hand_display_new (HAND_DISPLAY_MODE_CARD);
-	hand_display_set_style (card, handdisp->style, NULL);
+	hand_display_set_style (HAND_DISPLAY (card), handdisp->style);
 	hand_display_card_set_card (HAND_DISPLAY (card), handdisp->cur_drag);
 	gtk_container_add (GTK_CONTAINER (drag_win), card);
 	gtk_drag_set_icon_widget (dc, drag_win, 0, 0);
@@ -707,10 +732,12 @@ hand_display_drag_data_received (GtkWidget *hand, GdkDragContext *dc,
 	HandDisplay *handdisp = HAND_DISPLAY (hand);
 	int *card = (int *) selection_data->data;
 	int on_card = which_card(handdisp, x, y);
+	/*
 	if (*card == on_card) {
 		gtk_drag_finish (dc, FALSE, FALSE, t);
 		return;
 	}
+	*/
 	g_signal_emit_by_name (handdisp, "card-drag-drop", *card, on_card);
 	gtk_drag_finish (dc, TRUE, FALSE, t);
 }
@@ -739,13 +766,9 @@ hand_display_drag_end (GtkWidget *hand, GdkDragContext *dc, gpointer data)
 static void
 setup_dnd (HandDisplay *handdisp)
 {
-	GtkTargetEntry target_entry[1];
-	target_entry[0].target = "card";
-	target_entry[0].flags = 0;
-	target_entry[0].info = 0;
-
 	GtkWidget *hand = GTK_WIDGET (handdisp);
 
+	/* start manually
 	gtk_drag_source_set(
 		hand,
 		GDK_BUTTON1_MASK | GDK_BUTTON2_MASK,
@@ -753,6 +776,7 @@ setup_dnd (HandDisplay *handdisp)
 		1,
 		GDK_ACTION_COPY
 	);
+	*/
 	gtk_drag_dest_set(
 		hand,
 		GTK_DEST_DEFAULT_MOTION |
@@ -898,19 +922,30 @@ hand_display_draw (GtkWidget *hand)
 }
 
 void
-hand_display_set_style (HandDisplay *handdisp, int style, char *fname)
+hand_display_set_style (HandDisplay *handdisp, int style)
 {
-	char *card_fname = "/usr/share/pixmaps/gnome-games-common/cards/bonded.svg";
-	if (fname)
-		card_fname = fname;
 	handdisp->style = style;
-	if (style == HAND_DISPLAY_STYLE_CARDS && !render_init)
-		render_card_init (card_fname);
+}
+
+void
+hand_display_set_svg (char *fname, int width)
+{
+	assert (fname);
+	if (svg_filename && ! strcmp (fname, svg_filename) && width == card_width)
+		return; /* no change */
+
+	if (svg_filename)
+		g_free (svg_filename);
+	svg_filename = strdup (fname);
+	card_width = width;
+
+	render_card_init (svg_filename);
 }
 
 void
 hand_display_set_drag (HandDisplay *handdisp, int drag)
 {
+	handdisp->drag = drag;
 	if (drag) {
 		setup_dnd (handdisp);
 	}
