@@ -1,6 +1,6 @@
 /*
  *  tenace - bridge hand viewer and editor
- *  Copyright (C) 2005-2008 Christoph Berg <cb@df7cb.de>
+ *  Copyright (C) 2005-2009 Christoph Berg <cb@df7cb.de>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -28,6 +28,8 @@
 
 /* default paths */
 static char *svg_files[] = {
+	"bonded.svg",
+	"paris.svg",
 	"/usr/share/gnome-games-common/cards/bonded.svg", /* lenny */
 	"/usr/share/pixmaps/gnome-games-common/cards/bonded.svg", /* etch */
 	"/usr/share/gnome-games-common/cards/paris.svg", /* lenny */
@@ -46,12 +48,12 @@ window_options_board_populate (void) /* no parameter as it is called from window
 		return;
 
 	board *b = CUR_BOARD;
-	GtkWidget *w = glade_xml_get_widget (win->xml, "entry_title");
+	GtkWidget *w = get_widget ("entry_title");
 	if (b->name)
 		gtk_entry_set_text (GTK_ENTRY (w), b->name->str);
 	int i;
 	for (i = 0; i < 4; i++) {
-		w = glade_xml_get_widget (win->xml, entry_name[i]);
+		w = get_widget (entry_name[i]);
 		if (b->hand_name[i])
 			gtk_entry_set_text (GTK_ENTRY (w), b->hand_name[i]->str);
 	}
@@ -65,7 +67,7 @@ window_options_board_list_populate (void) /* no parameter as it is called from w
 	assert (board_store);
 
 	PROTECT_BEGIN;
-	GtkWidget *w = glade_xml_get_widget (win->xml, "board_list");
+	GtkWidget *w = get_widget ("board_list");
 	gtk_tree_view_set_model (GTK_TREE_VIEW (w), GTK_TREE_MODEL (board_store));
 
 	gtk_list_store_clear (board_store);
@@ -92,18 +94,18 @@ static void
 apply_options (GtkWidget *window_options)
 {
 	/* Card display */
-	GtkWidget *w = glade_xml_get_widget (win->xml, "show_played_cards");
+	GtkWidget *w = get_widget ("show_played_cards");
 	win->show_played_cards = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (w));
 
-	w = glade_xml_get_widget (win->xml, "show_as_cards");
+	w = get_widget ("show_as_cards");
 	int style = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (w)) ?
 		HAND_DISPLAY_STYLE_CARDS : HAND_DISPLAY_STYLE_TEXT;
 	board_window_set_style (win, style);
 	window_card_set_style (style);
 
-	w = glade_xml_get_widget (win->xml, "svg_file");
+	w = get_widget ("svg_file");
 	gchar *fname = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (w));
-	w = glade_xml_get_widget (win->xml, "spinbutton_card_width");
+	w = get_widget ("spinbutton_card_width");
 	win->card_width = gtk_spin_button_get_value (GTK_SPIN_BUTTON (w));
 
 	if (fname) {
@@ -115,15 +117,15 @@ apply_options (GtkWidget *window_options)
 
 	/* Hands */
 	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON
-		(glade_xml_get_widget (win->xml, "show_dd_all"))))
+		(get_widget ("show_dd_all"))))
 	{
 		win->show_dd_scores = seat_all;
 	} else if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON
-		(glade_xml_get_widget (win->xml, "show_dd_ns"))))
+		(get_widget ("show_dd_ns"))))
 	{
 		win->show_dd_scores = north_south;
 	} else if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON
-		(glade_xml_get_widget (win->xml, "show_dd_ew"))))
+		(get_widget ("show_dd_ew"))))
 	{
 		win->show_dd_scores = east_west;
 	} else {
@@ -131,11 +133,11 @@ apply_options (GtkWidget *window_options)
 	}
 
 	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON
-		(glade_xml_get_widget (win->xml, "show_hand_all"))))
+		(get_widget ("show_hand_all"))))
 	{
 		win->show_hands = seat_all;
 	} else if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON
-		(glade_xml_get_widget (win->xml, "show_hand_ns"))))
+		(get_widget ("show_hand_ns"))))
 	{
 		win->show_hands = north_south;
 	} else {
@@ -144,11 +146,11 @@ apply_options (GtkWidget *window_options)
 
 	/* Current board */
 	board *b = CUR_BOARD;
-	w = glade_xml_get_widget (win->xml, "entry_title");
+	w = get_widget ("entry_title");
 	g_string_printf (b->name, "%s", gtk_entry_get_text (GTK_ENTRY (w)));
 	int i;
 	for (i = 0; i < 4; i++) {
-		w = glade_xml_get_widget (win->xml, entry_name[i]);
+		w = get_widget (entry_name[i]);
 		g_string_printf (b->hand_name[i],
 			"%s", gtk_entry_get_text (GTK_ENTRY (w)));
 	}
@@ -308,7 +310,11 @@ write_config (window_board_t *win)
 
 	struct stat buf;
 	if (stat (g_get_user_config_dir (), &buf) == -1)
-		mkdir (g_get_user_config_dir (), 0777);
+		mkdir (g_get_user_config_dir ()
+#ifndef _WIN32
+				, 0777
+#endif
+				);
 
 	char rcfile[1024];
 	snprintf (rcfile, sizeof (rcfile), "%s/%s",
@@ -329,60 +335,59 @@ write_config (window_board_t *win)
 /* callbacks */
 
 /* create options window and fill it */
-void
+G_MODULE_EXPORT void
 on_options1_activate                   (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
 {
 	if (window_options)
 		return;
 
-	window_options = glade_xml_get_widget (win->xml, "window_options");
+	window_options = get_widget ("window_options");
 	gtk_widget_show (window_options);
 
 	/* Tab 1: Card display */
-	GtkWidget *w = glade_xml_get_widget (win->xml, "show_played_cards");
+	GtkWidget *w = get_widget ("show_played_cards");
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (w), win->show_played_cards);
 
-	w = glade_xml_get_widget (win->xml,
-		win->hand_display_style == HAND_DISPLAY_STYLE_CARDS ?
+	w = get_widget (win->hand_display_style == HAND_DISPLAY_STYLE_CARDS ?
 			"show_as_cards" : "show_as_text");
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (w), TRUE);
 
-	w = glade_xml_get_widget (win->xml, "svg_file");
+	w = get_widget ("svg_file");
 	if (win->svg)
 		gtk_file_chooser_set_filename (GTK_FILE_CHOOSER (w), win->svg);
 
-	w = glade_xml_get_widget (win->xml, "spinbutton_card_width");
+	w = get_widget ("spinbutton_card_width");
 	gtk_spin_button_set_value (GTK_SPIN_BUTTON (w), win->card_width);
 
 	/* Tab 2: Hands */
 	switch (win->show_hands) {
 		case seat_none: /* not yet implemented - is this useful? */
-			w = glade_xml_get_widget (win->xml, "show_hand_none");
+			w = get_widget ("show_hand_none");
 			break;
 		case east_west:
-			w = glade_xml_get_widget (win->xml, "show_hand_ew");
+			w = get_widget ("show_hand_ew");
 			break;
 		case north_south:
-			w = glade_xml_get_widget (win->xml, "show_hand_ns");
+			w = get_widget ("show_hand_ns");
 			break;
 		default:
-			w = glade_xml_get_widget (win->xml, "show_hand_all");
+			w = get_widget ("show_hand_all");
 	}
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (w), TRUE);
 
 	switch (win->show_dd_scores) {
 		case seat_none:
-			w = glade_xml_get_widget (win->xml, "show_dd_none");
+			w = get_widget ("show_dd_none");
 			break;
 		case east_west:
-			w = glade_xml_get_widget (win->xml, "show_dd_ew");
+			w = get_widget ("show_dd_ew");
 			break;
 		case north_south:
-			w = glade_xml_get_widget (win->xml, "show_dd_ns");
+			w = get_widget ("show_dd_ns");
 			break;
 		default:
-			w = glade_xml_get_widget (win->xml, "show_dd_all");
+			w = get_widget ("show_dd_all");
 	}
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (w), TRUE);
 
@@ -395,7 +400,7 @@ on_options1_activate                   (GtkMenuItem     *menuitem,
 			G_TYPE_INT, G_TYPE_STRING);
 	}
 
-	w = glade_xml_get_widget (win->xml, "board_list");
+	w = get_widget ("board_list");
 	gtk_tree_view_set_model (GTK_TREE_VIEW (w), GTK_TREE_MODEL (board_store));
 
 	GtkCellRenderer *renderer;
@@ -414,7 +419,7 @@ on_options1_activate                   (GtkMenuItem     *menuitem,
 }
 
 
-gboolean
+G_MODULE_EXPORT gboolean
 on_window_options_delete_event         (GtkWidget       *widget,
                                         GdkEvent        *event,
                                         gpointer         user_data)
@@ -424,7 +429,7 @@ on_window_options_delete_event         (GtkWidget       *widget,
 }
 
 
-void
+G_MODULE_EXPORT void
 on_options_cancel_clicked              (GtkButton       *button,
                                         gpointer         user_data)
 {
@@ -433,13 +438,14 @@ on_options_cancel_clicked              (GtkButton       *button,
 }
 
 
-void
+G_MODULE_EXPORT void
 on_options_generate_go_clicked         (GtkButton       *button,
                                         gpointer         user_data)
 {
-	GtkWidget *number = glade_xml_get_widget (win->xml, "options_generate_number");
+#ifndef _WIN32
+	GtkWidget *number = get_widget ("options_generate_number");
 	int n = gtk_spin_button_get_value (GTK_SPIN_BUTTON (number));
-	GtkWidget *view = glade_xml_get_widget (win->xml, "options_generate_entry");
+	GtkWidget *view = get_widget ("options_generate_entry");
 	GtkTextBuffer *buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (view));
 	GtkTextIter start, end;
 	gtk_text_buffer_get_iter_at_offset (buffer, &start, 0);
@@ -488,10 +494,11 @@ on_options_generate_go_clicked         (GtkButton       *button,
 	close (rfd[0]);
 
 	g_free (text);
+#endif
 }
 
 
-void
+G_MODULE_EXPORT void
 on_options_apply_clicked               (GtkButton       *button,
                                         gpointer         user_data)
 {
@@ -499,7 +506,7 @@ on_options_apply_clicked               (GtkButton       *button,
 }
 
 
-void
+G_MODULE_EXPORT void
 on_options_ok_clicked                  (GtkButton       *button,
                                         gpointer         user_data)
 {
