@@ -1,6 +1,6 @@
 /*
  *  tenace - bridge hand viewer and editor
- *  Copyright (C) 2005-2008 Christoph Berg <cb@df7cb.de>
+ *  Copyright (C) 2005-2009 Christoph Berg <cb@df7cb.de>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -186,56 +186,56 @@ void show_board (board *b, redraw_t redraw)
 	}
 
 	if (redraw & REDRAW_CONTRACT) {
-		w = glade_xml_get_widget (win->xml, "label_board");
+		w = get_widget ("label_board");
 		g_string_printf(str, "%s\n%s", b->name->str,
 			contract_string(b->level, b->trumps, b->declarer, b->doubled));
 		gtk_label_set_text((GtkLabel*) w, str->str);
 
 		char *dealermenu[] = { 0, "dealer_west1", "dealer_north1",
 			"dealer_east1", "dealer_south1"};
-		w = glade_xml_get_widget (win->xml, dealermenu[b->dealer]);
+		w = get_widget (dealermenu[b->dealer]);
 		gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (w), TRUE);
 
 		char *declarermenu[] = { 0, "declarer_west1", "declarer_north1",
 			"declarer_east1", "declarer_south1"};
-		w = glade_xml_get_widget (win->xml, declarermenu[b->declarer]);
+		w = get_widget (declarermenu[b->declarer]);
 		gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (w), TRUE);
 
 		char *levelmenu[] = { "level1" /* PASS */, "level1", "level2", "level3",
 			"level4", "level5", "level6", "level7" };
-		w = glade_xml_get_widget (win->xml, levelmenu[b->level]);
+		w = get_widget (levelmenu[b->level]);
 		gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (w), TRUE);
 
 		char *suitmenu[] = { "contract_clubs1", "contract_diamonds1",
 			"contract_hearts1", "contract_spades1", "contract_no_trump1" };
-		w = glade_xml_get_widget (win->xml, suitmenu[b->trumps]);
+		w = get_widget (suitmenu[b->trumps]);
 		gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (w), TRUE);
 
 		char *doublemenu[] = { "level_doubled0", "level_doubled1", "level_redoubled1" };
-		w = glade_xml_get_widget (win->xml, doublemenu[b->doubled]);
+		w = get_widget (doublemenu[b->doubled]);
 		gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (w), TRUE);
 
 		char *vulnmenu[] = { "vuln_none", "vuln_ns", "vuln_ew", "vuln_all" };
-		w = glade_xml_get_widget (win->xml, vulnmenu[2 * b->vuln[1] + b->vuln[0]]);
+		w = get_widget (vulnmenu[2 * b->vuln[1] + b->vuln[0]]);
 		gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (w), TRUE);
 
 		window_options_board_populate ();
 	}
 
 	if (redraw & REDRAW_NAMES) {
-		w = glade_xml_get_widget (win->xml, "label_west");
+		w = get_widget ("label_west");
 		board_set_player_name (w, west, b->dealer == west, b->vuln[1],
 				b->declarer == west,
 				b->current_turn == west, b->hand_name[0]->str);
-		w = glade_xml_get_widget (win->xml, "label_north");
+		w = get_widget ("label_north");
 		board_set_player_name (w, north, b->dealer == north, b->vuln[0],
 				b->declarer == north,
 				b->current_turn == north, b->hand_name[1]->str);
-		w = glade_xml_get_widget (win->xml, "label_east");
+		w = get_widget ("label_east");
 		board_set_player_name (w, east, b->dealer == east, b->vuln[1],
 				b->declarer == east,
 				b->current_turn == east, b->hand_name[2]->str);
-		w = glade_xml_get_widget (win->xml, "label_south");
+		w = get_widget ("label_south");
 		board_set_player_name (w, south, b->dealer == south, b->vuln[0],
 				b->declarer == south,
 				b->current_turn == south, b->hand_name[3]->str);
@@ -244,13 +244,13 @@ void show_board (board *b, redraw_t redraw)
 	}
 
 	if (redraw & REDRAW_TRICKS) {
-		w = glade_xml_get_widget (win->xml, "label_tricks");
+		w = get_widget ("label_tricks");
 		g_string_printf(str, _("NS: %d\nEW: %d"), b->tricks[0], b->tricks[1]);
 		gtk_label_set_markup((GtkLabel*) w, str->str);
 	}
 
 	if (redraw & REDRAW_PAR) {
-		w = glade_xml_get_widget (win->xml, "par_label");
+		w = get_widget ("par_label");
 		if (b->par_score == -1) {
 			gtk_label_set_text (GTK_LABEL (w), "");
 		} else {
@@ -509,7 +509,7 @@ static void create_hand_widgets (window_board_t *win)
 	GtkSizeGroup *sizegroup = gtk_size_group_new (GTK_SIZE_GROUP_BOTH);
 
 	for (h = 0; h < 4; h++) {
-		GtkWidget *alignment = glade_xml_get_widget (win->xml, alignment_a[h]);
+		GtkWidget *alignment = get_widget (alignment_a[h]);
 		GtkWidget *hand = hand_display_new(HAND_DISPLAY_MODE_HAND);
 		gtk_container_add(GTK_CONTAINER(alignment), hand);
 		gtk_widget_show(hand);
@@ -524,7 +524,7 @@ static void create_hand_widgets (window_board_t *win)
 	g_object_unref (sizegroup);
 
 	/* trick display */
-	GtkWidget *grid = glade_xml_get_widget (win->xml, "table1");
+	GtkWidget *grid = get_widget ("table1");
 	GtkWidget *table = hand_display_new (HAND_DISPLAY_MODE_TABLE);
 	gtk_table_attach_defaults (GTK_TABLE (grid), table, 1, 2, 1, 2);
 	gtk_widget_show (table);
@@ -544,13 +544,13 @@ create_bidding_widget (window_board_t *win)
 {
 	/*
 	GtkScrolledWindow *scroll = GTK_SCROLLED_WINDOW
-		(glade_xml_get_widget (win->xml, "scrolledwindow2"));
+		(get_widget ("scrolledwindow2"));
 	GdkColor bg = { 0, 0.8*65535, 0.0, 0.0 };
 	gdk_colormap_alloc_color (gdk_colormap_get_system (), &bg, FALSE, TRUE);
 	gtk_widget_modify_bg (GTK_WIDGET (scroll), GTK_STATE_NORMAL, &bg);
 	*/
 
-	win->bidding = GTK_TREE_VIEW (glade_xml_get_widget (win->xml, "treeview_bidding"));
+	win->bidding = GTK_TREE_VIEW (get_widget ("treeview_bidding"));
 	//gtk_widget_modify_bg (GTK_WIDGET (bidding), GTK_STATE_NORMAL, &bg);
 	//gdk_window_set_background (gtk_tree_view_get_bin_window (bidding), &bidding_vuln);
 	win->bidding_store = gtk_list_store_new (8,
@@ -636,14 +636,14 @@ void
 board_window_init (window_board_t *win)
 {
 	//win->window = create_window_hand ();
-	win->window = glade_xml_get_widget (win->xml, "window_hand");
-	win->statusbar = GTK_STATUSBAR (glade_xml_get_widget (win->xml, "statusbar1"));
-	win->board_menu = glade_xml_get_widget (win->xml, "board_menu1");
+	win->window = get_widget ("window_hand");
+	win->statusbar = GTK_STATUSBAR (get_widget ("statusbar1"));
+	win->board_menu = get_widget ("board_menu1");
 	create_hand_widgets(win);
 	create_bidding_widget (win);
 
 	/* set up "recently used" menu */
-	GtkWidget *jump_menu = glade_xml_get_widget (win->xml, "jump_to1");
+	GtkWidget *jump_menu = get_widget ("jump_to1");
 #if GTK_CHECK_VERSION (2,10,0)
 	GtkWidget *recentchooser = gtk_recent_chooser_menu_new ();
 	gtk_recent_chooser_set_sort_type (GTK_RECENT_CHOOSER (recentchooser),
@@ -682,7 +682,7 @@ board_window_init (window_board_t *win)
 	gtk_widget_show (win->window);
 }
 
-gboolean
+G_MODULE_EXPORT gboolean
 on_window_hand_delete_event            (GtkWidget       *widget,
                                         GdkEvent        *event,
                                         gpointer         user_data)
